@@ -557,6 +557,12 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProjectSingleConfig &tool)
     addFilters(tempProj, xmlFilter, "Deployment Files");
     addFilters(tempProj, xmlFilter, "Distribution Files");
 
+    tempProj.ExtraCompilers.reserve(tool.ExtraCompilersFiles.size());
+    std::transform(tool.ExtraCompilersFiles.cbegin(), tool.ExtraCompilersFiles.cend(),
+                   std::back_inserter(tempProj.ExtraCompilers),
+                   [] (const VCFilter &filter) { return filter.Name; });
+    tempProj.ExtraCompilers.removeDuplicates();
+
     for (int x = 0; x < tempProj.ExtraCompilers.count(); ++x)
         addFilters(tempProj, xmlFilter, tempProj.ExtraCompilers.at(x));
 
@@ -1977,6 +1983,15 @@ bool VCXProjectWriter::outputFileConfig(OutputFilterData *d, XmlOutput &xml, Xml
     return fileAdded;
 }
 
+static bool isFileClCompatible(const QString &filePath)
+{
+    auto filePathEndsWith = [&filePath] (const QString &ext) {
+        return filePath.endsWith(ext, Qt::CaseInsensitive);
+    };
+    return std::any_of(Option::cpp_ext.cbegin(), Option::cpp_ext.cend(), filePathEndsWith)
+            || std::any_of(Option::c_ext.cbegin(), Option::c_ext.cend(), filePathEndsWith);
+}
+
 void VCXProjectWriter::outputFileConfig(XmlOutput &xml, XmlOutput &xmlFilter,
                                         const QString &filePath, const QString &filterName)
 {
@@ -2000,7 +2015,7 @@ void VCXProjectWriter::outputFileConfig(XmlOutput &xml, XmlOutput &xmlFilter,
                       << attrTagS("Filter", filterName);
             xml << tag("ClInclude")
                 << attrTag("Include", nativeFilePath);
-        } else if (filePath.endsWith(".cpp")) {
+        } else if (isFileClCompatible(filePath)) {
             xmlFilter << tag("ClCompile")
                       << attrTag("Include", nativeFilePath)
                       << attrTagS("Filter", filterName);
